#!/usr/bin/ruby
require 'rtm'
require 'rtm_config'
require 'net/http'
require 'parsedate'

USER_AGENT = 'RMT/0.0.0.0'

def http_head(url, history = [])
  if URI::REGEXP::ABS_URI =~ url &&
    $1.downcase == 'http' && !$2
    host = $3
    port = $4 || 80
    path = $6 || '/'
    path << '?' + $7 if $7

    if history.include?([host, port, path]) || history.size > 5
      return [false]
    end
    history << [host, port, path]
    r = nil
    Net::HTTP.start(host, port) do |http|
      http.head2(path, {'user-agent' => USER_AGENT}) do |response|
        if ['200'].include?(response.code)
          r = [url, response]
        elsif ['301', '302'].include?(response.code) &&
            response.key?('location')
          r = http_head(response['location'], history)
        else
          r = [false, response]
        end
      end
    end

  else
    r = [false]
  end

  r
end

urldb  = PStore.new(RTM::Config['urldb'])
urldb.transaction do |db|
  db['urls'] = {} unless db.root?('urls')
  db['urls'].each do |url, data|
    last_last_modified = data[0]
    last_visited = data[1]
    now = Time.now

    print url + '... '
    if now - last_visited < 60*20
      print 'retry time not reached'

    else
      last_modified = nil
      rurl, r = http_head(url)
      if rurl && r['last-modified']
        print "\n -> " + rurl + '... ' if rurl != url
        last_modified = 
          Time.gm(*(ParseDate::parsedate(r['last-modified'])[0..5]))
        if last_modified > last_last_modified
          print 'updated at ' + last_modified.strftime('%Y-%m-%d %H:%M')
        else
          print 'done'
        end

      else
        puts 'hmm...  (' + r.code + ')'
        r.each{|k,v| puts ' ' + k + ': ' + v}
      end
      data.replace([last_modified || last_last_modified, now])
    end
    puts ''
  end
end
